/*
Plugin: jQuery.customSelect | Version: 1.0.0

Copyright (C) 2012 by Jamie Appleseed, Baymard Institute (baymard.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

(function( $ ){
	
	var defaults = {
		'initialized-class': 'custom-select-initialized',
		'focus-class': 'custom-select-focus',
		'hover-class': 'custom-select-hover',
		'disabled-class': 'custom-select-disabled',
		'custom-html': '<div class="custom-select-option"></div>',
		'select-selector': 'select:first'
	};
	
	
	var private_methods = {
		
		data: function( raw_instance ) {
			return $( raw_instance ).data( 'customSelect' );
		}
		
	};
	
	
	var public_methods = {
		
		init: function( customizations ) {
			return this.each(function(){
				
				var settings = $.extend( {}, defaults, customizations );
				var $that = $( this );
				
				if ( $that.data( 'customSelect' ) === undefined ) {
          
          var data = $that.data( 'customSelect', {
  					'settings': settings,
  					'$that': $that,
  					'$custom-select': $( settings['custom-html'] ),
  					'$select': $that.find( settings['select-selector'] )
  				}).data( 'customSelect' );
  				
  				data['$select']
  				  .after( data['$custom-select'] )
  				  .bind({
  				    'change.customSelect': function( event ) {
  				      data['$that'].customSelect( 'refresh' );
  				    },
  				    'mouseover.customSelect': function( event ) {
  				      data['$that'].addClass( data['settings']['hover-class'] );
  				    },
  				    'mouseleave.customSelect': function( event ) {
  				      data['$that'].removeClass( data['settings']['hover-class'] );
  				    },
  				    'focus.customSelect': function( event ) {
  				      data['$that'].addClass( data['settings']['focus-class'] );
  				      // Firefox only triggers 'change' event on blur
                data['$select'].bind( 'keypress.customSelect', function( event ) {
                  data['$select'].trigger( 'change.customSelect' );
                });
  				    },
  				    'blur.customSelect': function( event ) {
  				      data['$that'].removeClass( data['settings']['focus-class'] );
  				      data['$select'].unbind( 'keypress.customSelect' );
  				    }
  				  });
  				
  				data['$that']
  				  .customSelect( 'refresh' )
  				  .addClass( data['settings']['initialized-class'] );
				}
				
			});
		},
		
		refresh: function() {
			return this.each(function(){
				var data = private_methods['data']( this );
				
				if ( data['$select'].attr( 'disabled' ) ) {
				  data['$that'].addClass( data['settings']['disabled-class'] );
				} else {
				  data['$that'].removeClass( data['settings']['disabled-class'] );
				};
				
        data['$custom-select'].text( data['$that'].find( ':selected' ).text() );
        
			});
		},
		
		destroy: function() {
			return this.each(function(){
				var data = private_methods['data']( this );
				
        data['$custom-select'].remove();
        data['$select'].unbind( '.customSelect' );
	      data['$that']
	        .removeClass( data['settings']['initialized-class'] )
          .removeClass( data['settings']['disabled-class'] )
          .removeClass( data['settings']['hover-class'] )
          .removeClass( data['settings']['focus-class'] )
	        .removeData( 'customSelect' );
        
			});
		}
		
	};


	$.fn.customSelect = function( method ) {
		if ( public_methods[method] ) {
			return public_methods[method].apply( this, Array.prototype.slice.call( arguments, 1 ));
		} else if ( typeof method === 'object' || ! method ) {
			return public_methods.init.apply( this, arguments );
		} else {
			$.error( 'Method ' +  method + ' does not exist on jQuery.fn.customSelect' );
		}
	};
	
})( jQuery );
